(*
  UserInterface - Zaobalen funkc knihovny CommIntfDll.dll
  Vytvoeno pro Delphi 7

  Vydavatel :  Papouch s.r.o. - http://www.papouch.com/
  Autor     :  Martin Zmenk zamecnik@papouch.com

  popis     : objekt umouje komunikaci nejen se zazenmi firmy papouch s.r.o. pes sriov port, usb a ethernet (TCP nebo UDP)
              zaobaluje funkce knihovny CommIntfDll do objektu.

  Poznmky  : k pouit je poteba knihovna CommIntfDll.dll, a unita mapujc funkce uci_IDLLMapD.pas
              v ppad pouit pro USB jsou poteba t nainstalovan ovladae FTD direct drivers a knihova FTD2XX.dll, nebo nainstalovan virtuln sriov port

*)
unit uUserInterface;

interface
uses uci_IDllMapD,Windows,Messages,sysutils,classes;
//--- Registered messages
const
      RegMsg = WM_USER + 67;


type

  TUserCommunicationInterface = class  //-- Objekt komunikanho rozhran
  private
    fonDataRcv:TNotifyEvent;
    fonOpen:TNotifyEvent;
    fonClose:TNotifyEvent;
  protected
    fHandle:THandle;
    MyWnd:HWND;
    function GetHandle:THandle;
    function GetProvider:string;
    procedure WinProc(var Msg:TMessage);
    function GetOnOpen:TNotifyEvent;
    procedure SetOnOpen(const Value:TNotifyEvent);
    procedure DoOnOpen;
    function GetOnClose:TNotifyEvent;
    procedure SetOnClose(const Value:TNotifyEvent);
    procedure DoOnClose;
    procedure SetOnDataRcv(const Value:TNotifyEvent);
    function GetOnDataRcv:TNotifyEvent;
    procedure DoOnDataRcv;
    function GetActive: Boolean;
    procedure SetActive(const Value:Boolean); virtual;
    function GetValid:Boolean;
    procedure SetValid(const Value: Boolean);
  public
    constructor Create(ConnectionString:string);  //-- Konstruktor, pedv se pipojovac etzec
    //--       connection string je etzec pro vytvoen komunikanho rozhrann, uruje komunikan kanl a jeho nastaven (sriov port, tcp, udp, nebo ftd_usb)
    //   Tvorba connectionstringu : jsou to hodnoty ve tvaru "parametr=hodnota" oddleny stednkem. uruj typ pipojen
    //          nejdleitjm parametrem je "provider", uruje typ rozhrann. me to bt SERIAL_PORT, TCP_CLIENT, UDP_CLIENT nebo FTD_USB
    //          kad kanl pak me mt dal parametry. SERIAL_PORT (comport=slo portu;baudrate=rychlost - nemus bt zadno), TCP_CLIENT(host=ip adresa zazen;port=tcp port zazen), UDP_CLIENT(stejn jako TCP),
    //                                                    FTD_USB(index=poadov slo zazen; nebo device=st nzvu zazen; nebo serial_number=sriov slo zazen; nebo vid=vid;pid=pid; nebo nemus mt dn parametr, vezme se prvn pipojen zazen typu FTD_USB
    //          pklady: "provider=SERIAL_PORT;comport=1", "provider=TCP_CLIENT;host=192.168.1.254;port=10001", "provider=UDP_CLIENT;host=192.168.1.254;port=10001", "provider=FTD_USB;device=Quido"

    destructor Destroy;override;   //-- Destruktor
    property Handle:THandle read GetHandle;  //-- Originln Handle komunikanho rozhran
    property Provider:string read GetProvider;  //-- Typ rozhran (SERIAL_PORT, TCP_CLIENT, UDP_CLIENT nebo FTD_USB)

    property OnOpen:TNotifyEvent read GetOnOpen write SetOnOpen; //-- Udlost - pi oteven komunikanho kanlu rozhran
    property OnClose:TNotifyEvent read GetOnClose write SetOnClose; //-- Udlost - pi zaven komunikanho kanlu rozhran
    property OnDataRcv:TNotifyEvent read GetOnDataRcv write SetOnDataRcv; //-- Pi pjmu pchozch dat

    function GetConfiguration:String; //-- Vrt kofiguran etzec

    function GetId: String; //-- Vrt identifikaci rozhran
    function GetDescription:string; //-- Vrti popis rohran

    property Active:boolean read GetActive write setActive; //-- Lze zjistit i nastavit zda je rozhran oteveno i zaveno

    function Open: Boolean; //oteve komunikan rozhran (oteve sriov port, nave tcp spojen, ... podle typu komunikanho rozhran; nutno ped zapoetm komunikace;
    function Close(Force: Boolean=true): Boolean;  //zave komunikan rozhran

    function ReadString: String; //Pete pijat data
    function InBuffUsed: Cardinal; //Velikost pijatch dat (v bytech)
    function WriteString(str: String): Cardinal; //Odele data

    function SndRcv(const Data:string;MaxTimeOut:integer=500):string;
    //-- Odele data a ek na odpov. nvratov hodnota = pijat odpov
    function SndRcvSpin97(var Adr:byte;var Sig:byte;var Inst_Ack:byte;var Data:string;MaxTimeOut:integer=500):boolean;
         //-- Odele Spinel paket a pijme odpov. Vyslan paked se slo z adresy(Adr), podpisu(Sig),instrukce(Inst_Ack) a Dat(Data)
         // Pokud byla odpov pijata, nvratov hodnota je true, nastav pijatou adresu(Adr), podpis(Sig) a kd odpovdi Ack(Inst_Ack) a data se zmn na pijat Data
         // z vrcen hodnoty Ack (v Inst_Ack) lze urit, zda byl pkaz spn vykonn.
         // Pokud nebyla odpov pijata, nvratov hodnota je false.

    function ExecuteConfigDialog:Boolean; //Spust konfiguran dialog podle typu rozhran
    function ConfigDialogAvailable:Boolean; //Zjist, zda rozhran obsahuje konfiguran dialog

    class function ConnectionStringDlg(var ConnectionString:string;Caption:string=''):boolean; //-- Metoda tdy, kter vyvol dialog pro vytvoen pipojovacho etzce a vrt ho v parametru. vrac true, pokud bylo stisknuto OK
  end;

implementation

function TUserCommunicationInterface.Close(Force: Boolean): Boolean;
begin
  _ciClose(fHandle);
end;

function TUserCommunicationInterface.ConfigDialogAvailable: Boolean;
begin
  result:=ciConfigDialogAvailable(fHandle);
end;

class function TUserCommunicationInterface.ConnectionStringDlg(
  var ConnectionString: string; Caption: string): boolean;
begin
  if Caption='' then Caption:='Connection string';
  result:=_GetConnectionStringDlg(ConnectionString,Caption);
end;

constructor TUserCommunicationInterface.Create(ConnectionString: string);
begin
   if not CIDllOK then raise Exception.Create(CIDllInitMsg);
   inherited Create;
   MyWnd := AllocateHWnd(WinProc);
   fHandle := _ciCreate(ConnectionString,MyWnd,RegMsg);
   //ciRegisterNotifications(fHandle,MyWnd,RegMsg);
end;

destructor TUserCommunicationInterface.Destroy;
begin
 if fHandle<>0 then
 begin
  //ciClose(fHandle);
  ciRelease(fHandle);
 end;
 DeallocateHWnd(MyWnd);
 inherited;
end;

procedure TUserCommunicationInterface.DoOnClose;
begin
   if assigned(fOnClose) then fOnClose(self);
end;

procedure TUserCommunicationInterface.DoOnDataRcv;
begin
  if assigned(fOnDataRcv) then fonDataRcv(self);
end;

procedure TUserCommunicationInterface.DoOnOpen;
begin
  if assigned(fOnOpen) then fOnOpen(self);
end;

function TUserCommunicationInterface.ExecuteConfigDialog: Boolean;
begin
  result:=ciExecuteConfigDialog(fHandle);
end;

function TUserCommunicationInterface.GetActive: Boolean;
begin
  result:=_ciIsActive(fHandle);
end;

function TUserCommunicationInterface.GetConfiguration: String;
begin
end;

function TUserCommunicationInterface.GetDescription: string;
begin
  result:= _ciGetDescription(fHandle);
end;

function TUserCommunicationInterface.GetHandle: THandle;
begin
  result:=fHandle;
end;

function TUserCommunicationInterface.GetId: String;
begin
  result:=_ciGetID(fHandle);
end;

function TUserCommunicationInterface.GetOnClose: TNotifyEvent;
begin
  result:=fonClose;
end;

function TUserCommunicationInterface.GetOnDataRcv: TNotifyEvent;
begin
  result:=fonDataRcv;
end;

function TUserCommunicationInterface.GetOnOpen: TNotifyEvent;
begin
 result:=fonOpen;
end;

function TUserCommunicationInterface.GetProvider: string;
begin
  result:=_ciGetProvider(fHandle);
end;

function TUserCommunicationInterface.GetValid: Boolean;
begin
  result:=_ciGetValid(fHandle);
end;

function TUserCommunicationInterface.InBuffUsed: Cardinal;
begin
  result:= ciReadLength(fHandle);
end;

function TUserCommunicationInterface.Open: Boolean;
begin
 _ciOpen(fHandle);
end;

function TUserCommunicationInterface.ReadString: String;
begin
  result := _ciRead(fHandle);
end;

procedure TUserCommunicationInterface.SetActive(const Value: Boolean);
begin
  if Active then Close(true) else Open;
end;

procedure TUserCommunicationInterface.SetOnClose(
  const Value: TNotifyEvent);
begin
 fonClose := Value;
end;

procedure TUserCommunicationInterface.SetOnDataRcv(
  const Value: TNotifyEvent);
begin
  fonDataRcv := Value;
end;

procedure TUserCommunicationInterface.SetOnOpen(const Value: TNotifyEvent);
begin
  fOnopen := Value;
end;

procedure TUserCommunicationInterface.SetValid(const Value: Boolean);
begin
  _ciSetValid(fHandle,Value);
end;

function TUserCommunicationInterface.SndRcv(const Data: string;
  MaxTimeOut: integer): string;
begin
 result:=_SndRcv(fHandle,Data,MaxTimeOut);
end;

function TUserCommunicationInterface.SndRcvSpin97(var Adr, Sig,
  Inst_Ack: byte; var Data: string; MaxTimeOut: integer): boolean;
begin
 result:=_SndRcvSpin97(fHandle,Adr,Sig,Inst_Ack,Data,MaxTimeOut);
end;

procedure TUserCommunicationInterface.WinProc(var Msg: TMessage);
begin
  if (Msg.Msg = RegMsg)and(Msg.WParam = fHandle) then
  begin
    case Msg.LParam of
      ciev_Open : DoOnOpen;
      ciev_Close : DoOnClose;
      ciev_DataRcv: DoOnDataRcv;
    end;
  end;
end;

function TUserCommunicationInterface.WriteString(str: String): Cardinal;
begin
  result:=length(str);
  _ciWrite(fHandle,str);
end;


end.
